@@ -1,5 +1,5 @@
 import asyncio
-from typing import Callable, Dict, Tuple
+from typing import Callable, Generic, Tuple, TypeVar
 
 from omnipy.api.types import GeneralDecorator
 from omnipy.compute.job import JobBase
@@ -9,13 +9,14 @@ from omnipy.compute.mixins.params import ParamsFuncJobBaseMixin
 from omnipy.compute.mixins.result_key import ResultKeyFuncJobBaseMixin
 from omnipy.compute.mixins.serialize import SerializerFuncJobBaseMixin
 
+C = TypeVar('C', bound=Callable)
 
-class PlainFuncArgJobBase(JobBase):
-    def __init__(self, job_func: Callable, *args: object, **kwargs: object) -> None:
-        # JobBase.__init__(self, job_func, *args, **kwargs)
+
+class PlainFuncArgJobBase(JobBase, Generic[C]):
+    def __init__(self, job_func: C, *args: object, **kwargs: object) -> None:
         self._job_func = job_func
 
-    def _get_init_args(self) -> Tuple[object, ...]:
+    def _get_init_args(self) -> Tuple[C]:
         return self._job_func,
 
     def has_coroutine_func(self) -> bool:
@@ -34,11 +35,11 @@ class PlainFuncArgJobBase(JobBase):
         return self._job_func(*args, **kwargs)
 
     def _accept_call_func_decorator(self, call_func_decorator: GeneralDecorator) -> None:
-        self._call_func = call_func_decorator(self._call_func)
+        self._call_func = call_func_decorator(self._call_func)  # type:ignore
 
 
 # Extra level needed for mixins to be able to overload _call_job (and possibly other methods)
-class FuncArgJobBase(PlainFuncArgJobBase):
+class FuncArgJobBase(PlainFuncArgJobBase[C], Generic[C]):
     ...
 
 
@@ -47,29 +48,3 @@ FuncArgJobBase.accept_mixin(IterateFuncJobBaseMixin)
 FuncArgJobBase.accept_mixin(SerializerFuncJobBaseMixin)
 FuncArgJobBase.accept_mixin(ResultKeyFuncJobBaseMixin)
 FuncArgJobBase.accept_mixin(ParamsFuncJobBaseMixin)
-
-# class FuncJobTemplate(FuncArgJobBase, JobTemplate, ABC):
-#     def refine(self,
-#                update: bool = True,
-#                name: Optional[str] = None,
-#                iterate_over_data_files: bool = False,
-#                fixed_params: Optional[Mapping[str, object]] = None,
-#                param_key_map: Optional[Mapping[str, str]] = None,
-#                result_key: Optional[str] = None,
-#                persist_outputs: Optional[PersistOutputsOptions] = None,
-#                restore_outputs: Optional[RestoreOutputsOptions] = None,
-#                **kwargs: object):  # -> FuncJobTemplateT:
-#
-#         return self._refine(
-#             self,
-#             update=update,
-#             **remove_none_vals(
-#                 name=name,
-#                 iterate_over_data_files=iterate_over_data_files,
-#                 fixed_params=fixed_params,
-#                 param_key_map=param_key_map,
-#                 result_key=result_key,
-#                 persist_outputs=persist_outputs,
-#                 restore_outputs=restore_outputs,
-#                 **kwargs,
-#             ))
