@@ -1,26 +1,22 @@
 from functools import update_wrapper
 from types import MethodWrapperType
-from typing import Callable, cast, Protocol, runtime_checkable, Type, TypeVar
+from typing import Callable, cast, Protocol, Type, TypeVar
 
-# Types
+from omnipy.api.protocols.private.util import IsCallableClass, IsCallableParamAfterSelf
+from omnipy.api.types import DecoratorClassT
 
-DecoratorClassT = TypeVar('DecoratorClassT', covariant=True)
+CC = TypeVar('CC', bound=Callable, contravariant=True)
+
+C = TypeVar('C', bound=Callable)
 
 
-@runtime_checkable
-class CallableParamAfterSelf(Protocol):
-    """"""
-    def __call__(self, callable_arg: Callable, /, *args: object, **kwargs: object) -> None:
+class IsDecoratorClass(Protocol[CC]):
+    def __call__(self, func: CC, *args: object, **kwargs: object) -> None:
         ...
 
 
-@runtime_checkable
-class CallableClass(Protocol[DecoratorClassT]):
-    def __call__(self, *args: object, **kwargs: object) -> Callable[[Callable], DecoratorClassT]:
-        ...
-
-
-def callable_decorator_cls(cls: Type[DecoratorClassT]) -> CallableClass[DecoratorClassT]:
+def callable_decorator_cls(
+        cls: Type[IsDecoratorClass[CC]]) -> IsCallableClass[IsDecoratorClass[CC], CC]:
     """
     "Meta-decorator" that allows any class to function as a decorator for a callable.
 
@@ -34,7 +30,7 @@ def callable_decorator_cls(cls: Type[DecoratorClassT]) -> CallableClass[Decorato
         cls._wrapped_call: Callable = cast(Callable, cls.__call__)
 
     def _forward_call_to_obj_if_callable(self, *args: object,
-                                         **kwargs: object) -> Type[DecoratorClassT]:
+                                         **kwargs: object) -> Type[IsDecoratorClass[CC]]:
         """
         __call__ method at the class level which forward the call to instance-level call methods,
         if present (hardcoded as '_obj_call()'). This is needed due to the peculiarity that Python
@@ -70,26 +66,27 @@ def callable_decorator_cls(cls: Type[DecoratorClassT]) -> CallableClass[Decorato
 
     _wrapped_new: Callable = cls.__new__
 
-    def _new_wrapper(cls, *args: object, **kwargs: object) -> DecoratorClassT:
+    def _new_wrapper(cls, *args: object, **kwargs: object) -> IsDecoratorClass[CC]:
         if _wrapped_new is object.__new__:
             obj = _wrapped_new(cls)
         else:
             obj = _wrapped_new(cls, *args, **kwargs)
             # setattr(cls, '__new__', _wrapped_new)
 
-        _wrapped_init: CallableParamAfterSelf = cast(CallableParamAfterSelf, obj.__class__.__init__)
+        _wrapped_init: IsCallableParamAfterSelf = cast(IsCallableParamAfterSelf,
+                                                       obj.__class__.__init__)
 
         # Wrapper method that replaces the __init__ method of the decorated class
         def _init_wrapper(self, *args: object, **kwargs: object) -> None:
             args_list = list(args)
 
-            def _init(callable_arg: Callable) -> None:
+            def _init(callable_arg: CC) -> None:
                 _wrapped_init(self, callable_arg, *args_list, **kwargs)
                 update_wrapper(self, callable_arg, updated=[])
 
             if len(args_list) == 1 and _real_callable(args_list[0]):
                 # Decorate the callable directly
-                _callable_arg: Callable = cast(Callable, args_list[0])
+                _callable_arg = cast(CC, args_list[0])
                 args_list.pop(0)
                 _init(_callable_arg)
             else:
@@ -97,7 +94,7 @@ def callable_decorator_cls(cls: Type[DecoratorClassT]) -> CallableClass[Decorato
                 # class-level __call__ method. When this method is called, the provided _callable_arg
                 # is decorated.
                 def _init_as_obj_call_method(
-                        self, _callable_arg: Callable) -> Type[DecoratorClassT]:  # noqa
+                        self, _callable_arrg: CC) -> Type[IsDecoratorClass[CC]]:  # noqa
                     _init(_callable_arg)
                     del self._obj_call
                     return self
@@ -112,4 +109,4 @@ def callable_decorator_cls(cls: Type[DecoratorClassT]) -> CallableClass[Decorato
 
     setattr(cls, '__new__', _new_wrapper)
 
-    return cast(CallableClass[DecoratorClassT], cls)
+    return cast(IsCallableClass[IsDecoratorClass[CC], CC], cls)
